use strict;
use warnings;

package WebService::Shippo::Entities;
require WebService::Shippo::Object;
require WebService::Shippo::Collection;
require WebService::Shippo::Resource;
require WebService::Shippo::Addresses;
require WebService::Shippo::Config;
require WebService::Shippo::CarrierAccounts;
require WebService::Shippo::CustomsDeclarations;
require WebService::Shippo::CustomsItems;
require WebService::Shippo::Manifests;
require WebService::Shippo::Parcels;
require WebService::Shippo::Rates;
require WebService::Shippo::Refunds;
require WebService::Shippo::Shipments;
require WebService::Shippo::Transactions;

1;
